#pragma once

#include "TipApplication.h"
#include "UpgraderModel.h"
#include "UpgraderView.h"
#include "TipControl.h"

#include <atlpath.h>

//#include "Command/InitCommand.h"
//#include "../../../common/ConfigXmlParser.h"

namespace tipupgrader
{
typedef std::basic_string<TCHAR> tstring;
class UpgraderMVCFactory : public IMVCFactory
{
public:
	UpgraderMVCFactory() {}
	virtual ~UpgraderMVCFactory() {}

	//virtual override
	AbstractModel* CreateModel()
	{
		return new UpgraderModel;
	}
	AbstractView* CreateView()
	{
		return new UpgraderView;
	}
	AbstractControl* CreateControl()
	{
		return new TipControl;
	}

	void DestroyModel(AbstractModel* model)
	{
		if (model != NULL)
			delete model;
	}
	void DestroyView(AbstractView* view)
	{
		if (view != NULL)
			delete view;
	}
	void DestroyControl(AbstractControl* control)
	{
		if (control != NULL)
			delete control;
	}
	BOOL GetResourcePath(TCHAR* path, int lenPath)
	{
		TCHAR szApplicationPath[_MAX_PATH] = {0};
		GetAppRootDir(szApplicationPath, _MAX_PATH);
		ATLPath::RemoveFileSpec(szApplicationPath);

		TCHAR szResourcePath[_MAX_PATH] = {0};
		_tcsncpy_s(szResourcePath, szApplicationPath, _MAX_PATH);

		Application* app = Application::get_instance();
/*
		UpgraderModel* model = dynamic_cast<UpgraderModel*>(app->GetModel());
		TipAppDataManager* appData = model->GetAppDataManager();
		CString resourceFileName = appData->GetProperty(tipcommon::Element::ResourceFileName);
*/
// 		TCHAR resourceFileName[tipbase::MAXURL] = {0, };
// 		const TCHAR CONFIG_XML_FILENAME[] = _T("NToolsUpgrader.xml");
// 		if (FALSE == tipcommon::ConfigXmlParser::GetURL(CONFIG_XML_FILENAME, tipcommon::Element::ResourceFileName, resourceFileName, tipbase::MAXURL))
// 			return FALSE;

		ATLPath::Append(szResourcePath, _T("NtalkUpdaterResource_ko-KR.zip"));

		_tcsncpy_s(path, lenPath, szResourcePath, _MAX_PATH);
		return TRUE;
	}

	BOOL GetAppRootDir(TCHAR* pDir, int nMax)
	{
		ZeroMemory(pDir, nMax);
		if(GetModuleFileName( NULL, pDir, nMax))
		{
			TCHAR szDrive[MAX_PATH] = {0}; 
			TCHAR szPath[MAX_PATH] = {0}; 

#ifdef UNICODE
			_wsplitpath(pDir, szDrive, szPath, NULL, NULL);
#else
			_splitpath(pDir, szDrive, szPath, NULL, NULL); 
#endif
			ZeroMemory(pDir, nMax);
			_tcscpy(pDir, szDrive);
			_tcscat(pDir, szPath);
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}
};

class UpgraderApplication : public TipApplication
{
public:
	UpgraderApplication(std::vector<CString>* cmdParameter):m_MVCFactory(new UpgraderMVCFactory()),m_CmdParameter(cmdParameter)
	{
		m_returnValue = IDOK;
		m_bEnableCoreValueCheck = TRUE;
	}
	virtual ~UpgraderApplication(void) {}
	
	BOOL IsSilentRun()
	{
		BOOL bSilentRun = FALSE;
		if (m_CmdParameter->size())
		{
			int num = m_CmdParameter->size();
			CString silentParameter;
			for (int i = 0; i < num; i++)
			{
				silentParameter = m_CmdParameter->at(i);
				if (0 == silentParameter.CompareNoCase(_T("/M")))
					bSilentRun = TRUE;
			}
		}
		return bSilentRun;
	}

	BOOL IsValidRun()
	{
		return TRUE;
	}

	void SetAgreementUrl(LPCTSTR pViewUrl, LPCTSTR pLinkUrl1, LPCTSTR pLinkUrl2 = NULL)
	{
		m_strAgreementUrl[0].clear();
		m_strAgreementUrl[1].clear();
		m_strAgreementUrl[2].clear();

		if (pViewUrl)
			m_strAgreementUrl[0] = pViewUrl;
		if (pLinkUrl1)
			m_strAgreementUrl[1] = pLinkUrl1;
		if (pLinkUrl2)
			m_strAgreementUrl[2] = pLinkUrl2;
	}
	tstring GetAgreementUrl(int index) 
	{
		return m_strAgreementUrl[index];
	}

	UINT_PTR	GetReturnValue()
	{
		return m_returnValue;
	}
	BOOL		GetCoreValueCheck()
	{
		return m_bEnableCoreValueCheck;
	}
private:

	tstring m_strAgreementUrl[3];
	UINT_PTR	m_returnValue;
	BOOL		m_bEnableCoreValueCheck;

protected:
	IMVCFactory* GetMVCFactory() { return m_MVCFactory.get(); }
	virtual BOOL PreMessageLoop();

private:
	std::auto_ptr<UpgraderMVCFactory>	m_MVCFactory;	
	std::vector<CString>*				m_CmdParameter;

	
};

};